==> xl-repository-jcr-2015.5.0-sources.jar: com/xebialabs/deployit/repository/SearchQueryBuilder.java <==
package com.xebialabs.deployit.repository;

import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.base.Strings;
import com.google.common.collect.Collections2;
import com.google.common.collect.Lists;
import com.xebialabs.deployit.checks.Checks;
import com.xebialabs.deployit.jcr.JcrQueryTemplate;
import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.reflect.DescriptorRegistry;
import com.xebialabs.deployit.plugin.api.reflect.PropertyDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.collect.Lists.newArrayList;
import static com.xebialabs.deployit.jcr.JcrConstants.*;
import static com.xebialabs.deployit.plugin.api.reflect.PropertyKind.*;
import static com.xebialabs.deployit.repository.JcrPathHelper.getAbsolutePathFromId;
import static java.lang.String.format;

/**
 */
public class SearchQueryBuilder {
    public static final String CI_SELECTOR_NAME = "ci";

    private static final String EQUALITY_OPERATOR = "=";
    private static final String LIKE_OPERATOR = "LIKE";
    private static final String NAME_IS_OPERATOR = "NAME_IS";
    private static final String NAME_LIKE_OPERATOR = "NAME_LIKE";
    private static final String ISCHILDNODE_OPERATOR = "ISCHILDNODE";
    private static final String ISDESCENDANTNODE_OPERATOR = "ISDESCENDANTNODE";
    private static final String CONTAINS_OPERATOR = "CONTAINS";

    private final SearchParameters parameters;
    private final StringBuilder joins;
    private final List<Condition> conditions;
    private int nextSelectorId = 1;

    public SearchQueryBuilder(final SearchParameters parameters) {
        this.parameters = parameters;
        this.joins = new StringBuilder();
        this.conditions = newArrayList();

        Checks.checkArgument(parameters.getName() == null || !parameters.getName().contains("\'"), "Name [%s] contains a single quote (\')", parameters.getName());
        Checks.checkArgument(parameters.getParent() == null || !parameters.getParent().contains("\'"), "Parent [%s] contains a single quote (\')", parameters.getParent());
    }

    protected QueryTemplate createTemplate() {
        return createTemplate(new QueryTemplateFactory() {

            @Override
            public QueryTemplate createTemplate(String query) {
                return new JcrQueryTemplate(query);
            }

            @Override
            public QueryTemplate createTemplate(String query, Map<String, Object> parameters) {
                return new JcrQueryTemplate(query, parameters);
            }
        });
    }

    protected QueryTemplate createTemplate(QueryTemplateFactory factory) {
        createJoinsAndConditions();

        String queryString = constructQueryString();
        QueryTemplate template = factory.createTemplate(queryString);
        addParameters(template);
        template.setResultsPerPage(parameters.getResultsPerPage());
        template.setPage(parameters.getPage());
        template.setDepth(parameters.getDepth());

        return template;
    }

    private String constructQueryString() {
        StringBuilder builder = new StringBuilder();
        builder.append("SELECT " + CI_SELECTOR_NAME + ".* FROM [" + CONFIGURATION_ITEM_NODETYPE_NAME + "] AS " + CI_SELECTOR_NAME);
        builder.append(joins.toString());

        if (!conditions.isEmpty()) {
            builder.append(" WHERE ");
            builder.append(Joiner.on(" AND ").join(Collections2.transform(conditions, new Function<Condition, String>() {
                @Override
                public String apply(final Condition input) {
                    return input.build();
                }
            })));
        }

        builder.append(" ORDER BY NAME(" + CI_SELECTOR_NAME + ")");

        logger.trace("Query string built: {}", builder.toString());

        return builder.toString();
    }

    private void addParameters(QueryTemplate query) {
        for (Condition condition : conditions) {
            if (condition.parameter == null)
                continue;

            for (int i = 0; i < condition.values.length; i++) {
                String key = condition.parameter + i;
                Object value = condition.values[i];

                query.setParameter(key, value);
                logger.trace("Bound {} to {}", key, value);
            }
        }
    }

    private void createJoinsAndConditions() {
        createConditionForParent();
        createConditionForAncestor();
        createConditionForName();
        createConditionForConfigurationItemTypeName();
        createConditionForDate();
        createConditionsForProperties();
        createConditionsForTextSearch();
    }

    private void createConditionForConfigurationItemTypeName() {
        if (parameters.getType() != null) {
            List<String> types = Lists.newArrayList();
            types.add(parameters.getType().toString());
            for (Type subtype : DescriptorRegistry.getSubtypes(parameters.getType())) {
                types.add(subtype.toString());
            }
            conditions.add(Condition.from(CI_SELECTOR_NAME, CONFIGURATION_ITEM_TYPE_PROPERTY_NAME, "_configurationItemTypeName", EQUALITY_OPERATOR, false,
                    (Object[]) types.toArray(new Object[types.size()])));
        }
    }

    private void createConditionForParent() {
        if (!Strings.nullToEmpty(parameters.getParent()).trim().isEmpty()) {
            String parent = Strings.nullToEmpty(parameters.getParent());
            if (!parent.startsWith("/")) {
                parent = "/" + parent;
            }
            conditions.add(Condition.isChildNode(CI_SELECTOR_NAME, parent));
        }
    }

    private void createConditionForAncestor() {
        if (!Strings.nullToEmpty(parameters.getAncestor()).trim().isEmpty()) {
            String ancestor = Strings.nullToEmpty(parameters.getAncestor());
            if (!ancestor.startsWith("/")) {
                ancestor = "/" + ancestor;
            }
            conditions.add(Condition.isDescendantNode(CI_SELECTOR_NAME, ancestor));
        }
    }

    private void createConditionForName() {
        if (!Strings.nullToEmpty(parameters.getName()).trim().isEmpty()) {
            if (parameters.getName().contains("%")) {
                conditions.add(Condition.nameLike(CI_SELECTOR_NAME, parameters.getName()));
            } else {
                conditions.add(Condition.nameIs(CI_SELECTOR_NAME, parameters.getName()));
            }
        }
    }

    private void createConditionForDate() {
        if (parameters.getBefore() != null) {
            conditions.add(Condition.from(CI_SELECTOR_NAME, LAST_MODIFIED_AT_PROPERTY_NAME, "_before", "<=", parameters.getBefore()));
        }
    }

    private void createConditionsForProperties() {
        for (Map.Entry<String, String> entry : parameters.getProperties().entrySet()) {
            createConditionForProperty(entry.getKey(), entry.getValue());
        }
    }

    private void createConditionForProperty(final String propertyName, final String propertyValue) {
        if (parameters.getType() != null) {
            Descriptor descriptor = DescriptorRegistry.getDescriptor(parameters.getType());
            if (descriptor != null) {
                PropertyDescriptor propertyDescriptor = descriptor.getPropertyDescriptor(propertyName);
                if (propertyDescriptor != null) {
                    if (propertyDescriptor.getKind() == CI) {
                        String selectorName = "referenced" + nextSelectorId;
                        joins.append(" INNER JOIN [" + CONFIGURATION_ITEM_NODETYPE_NAME + "] AS " + selectorName + " ON " + CI_SELECTOR_NAME + ".["
                                + propertyName + "] = " + selectorName + ".[jcr:uuid]");
                        int lastSlash = propertyValue.lastIndexOf('/');
                        checkArgument(lastSlash != -1, propertyValue + " is a ID but does not contain a slash (/)");
                        final String parent = getAbsolutePathFromId(propertyValue.substring(0, lastSlash));
                        final String name = propertyValue.substring(lastSlash + 1);
                        conditions.add(Condition.isChildNode(selectorName, parent));
                        conditions.add(Condition.nameIs(selectorName, name));
                        nextSelectorId++;
                        return;
                    } else if (propertyDescriptor.getKind() == SET_OF_CI || propertyDescriptor.getKind() == LIST_OF_CI) {
                        // FIXME: Find out how to make the JOIN works on this
                        // case. The INNER JOIN used for the single CI property
                        // causes unpredictable results.
                        // The results of the JOIN are different every time if
                        // the multi-value property has more than one value.
                        throw new IllegalArgumentException("Cannot query property " + propertyDescriptor + " because it is of type SET_OF_CIS");
                    }
                }
            }
        }
        conditions.add(Condition.match(CI_SELECTOR_NAME, propertyName, propertyName, propertyValue));
    }

    private void createConditionsForTextSearch() {
        int count = 0;
        for (Map.Entry<String, String> query : parameters.getTextSearchQueries().entrySet()) {
            conditions.add(Condition.textSearch(query.getKey(), query.getValue(), "textSearch" + count + "_"));
        }
    }

    private static class Condition {
        String selector;
        String field;
        String operator;
        String parameter;
        Object[] values;
        boolean caseInsensitive;

        private static final String NAME_IS_FORMAT = "NAME(%s) = \'%s\'";
        private static final String NAME_LIKE_FORMAT = "LOWER(NAME(%s)) LIKE \'%s\'";
        private static final String ISCHILDNODE_FORMAT = ISCHILDNODE_OPERATOR + "(%s, [\'%s\'])";
        private static final String ISDESCENDANTNODE_FORMAT = ISDESCENDANTNODE_OPERATOR + "(%s, [\'%s\'])";
        private static final String CONTAINS_FORMAT = "contains(%s.[%s], $%s)";
        private static final String CASE_SENSITIVE = "%s.[%s] %s $%s";
        private static final String CASE_INSENSITIVE = "LOWER(%s.[%s]) %s $%s";

        String build() {
            StringBuilder conditionString = new StringBuilder();
            conditionString.append("(");
            for (int i = 0; i < values.length; i++) {
                if (i > 0) {
                    conditionString.append(" OR ");
                }

                if (operator.equals(NAME_IS_OPERATOR)) {
                    conditionString.append(format(NAME_IS_FORMAT, selector, values[i]));
                } else if (operator.equals(NAME_LIKE_OPERATOR)) {
                    conditionString.append(format(NAME_LIKE_FORMAT, selector, values[i]));
                } else if (operator.equals(ISCHILDNODE_OPERATOR)) {
                    conditionString.append(format(ISCHILDNODE_FORMAT, selector, values[i]));
                } else if (operator.equals(ISDESCENDANTNODE_OPERATOR)) {
                    conditionString.append(format(ISDESCENDANTNODE_FORMAT, selector, values[i]));
                } else if (operator.equals(CONTAINS_OPERATOR)) {
                    conditionString.append(format(CONTAINS_FORMAT, selector, field, parameter + i));
                } else {
                    conditionString.append(format(caseInsensitive ? CASE_INSENSITIVE : CASE_SENSITIVE, selector, field, operator, parameter + i));
                }
            }
            conditionString.append(")");
            return conditionString.toString();
        }

        static Condition match(final String selector, final String field, final String parameter, final String value) {
            String operator;
            boolean caseInsensitive;
            if (value.contains("%")) {
                operator = LIKE_OPERATOR;
                caseInsensitive = true;
            } else {
                operator = EQUALITY_OPERATOR;
                caseInsensitive = false;
            }
            return from(selector, field, parameter, operator, caseInsensitive, value);
        }

        static Condition nameIs(final String selector, final String value) {
            return from(selector, null, null, NAME_IS_OPERATOR, false, value);
        }

        static Condition nameLike(final String selector, final String value) {
            return from(selector, null, null, NAME_LIKE_OPERATOR, false, value.toLowerCase());
        }

        static Condition isChildNode(final String selector, final String value) {
            return from(selector, null, null, ISCHILDNODE_OPERATOR, false, value);
        }

        static Condition isDescendantNode(final String selector, final String value) {
            return from(selector, null, null, ISDESCENDANTNODE_OPERATOR, false, value);
        }

        static Condition textSearch(String property, String textQuery, String parameterName) {
            return Condition.from(CI_SELECTOR_NAME, property, parameterName, CONTAINS_OPERATOR, false, textQuery);
        }

        static Condition from(final String selector, final String field, final String parameter, final String operator, final Object value) {
            return from(selector, field, parameter, operator, false, value);
        }

        static Condition from(final String selector, final String field, final String parameter, final String operator, final boolean caseInsensitive,
                final Object... values) {
            Condition t = new Condition();
            t.selector = selector;
            t.field = field;
            t.parameter = parameter;
            t.operator = operator;
            t.caseInsensitive = caseInsensitive;
            t.values = values;
            return t;
        }
    }

    private static final Logger logger = LoggerFactory.getLogger(SearchQueryBuilder.class);
}

==> xl-repository-jcr-2015.6.4-sources.jar: com/xebialabs/deployit/repository/SearchQueryBuilder.java <==
package com.xebialabs.deployit.repository;

import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.base.Strings;
import com.google.common.collect.Collections2;
import com.google.common.collect.Lists;
import com.xebialabs.deployit.jcr.JcrQueryTemplate;
import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.reflect.DescriptorRegistry;
import com.xebialabs.deployit.plugin.api.reflect.PropertyDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.collect.Lists.newArrayList;
import static com.xebialabs.deployit.jcr.JcrConstants.*;
import static com.xebialabs.deployit.plugin.api.reflect.PropertyKind.*;
import static com.xebialabs.deployit.repository.JcrPathHelper.getAbsolutePathFromId;
import static java.lang.String.format;

/**
 */
public class SearchQueryBuilder {
    public static final String CI_SELECTOR_NAME = "ci";

    private static final String EQUALITY_OPERATOR = "=";
    private static final String LIKE_OPERATOR = "LIKE";
    private static final String NAME_IS_OPERATOR = "NAME_IS";
    private static final String NAME_LIKE_OPERATOR = "NAME_LIKE";
    private static final String ISCHILDNODE_OPERATOR = "ISCHILDNODE";
    private static final String ISDESCENDANTNODE_OPERATOR = "ISDESCENDANTNODE";
    private static final String CONTAINS_OPERATOR = "CONTAINS";

    private final SearchParameters parameters;
    private final StringBuilder joins;
    private final List<Condition> conditions;
    private int nextSelectorId = 1;

    public SearchQueryBuilder(final SearchParameters parameters) {
        this.parameters = parameters;
        this.joins = new StringBuilder();
        this.conditions = newArrayList();
    }

    protected QueryTemplate createTemplate() {
        return createTemplate(new QueryTemplateFactory() {

            @Override
            public QueryTemplate createTemplate(String query) {
                return new JcrQueryTemplate(query);
            }

            @Override
            public QueryTemplate createTemplate(String query, Map<String, Object> parameters) {
                return new JcrQueryTemplate(query, parameters);
            }
        });
    }

    protected QueryTemplate createTemplate(QueryTemplateFactory factory) {
        createJoinsAndConditions();

        String queryString = constructQueryString();
        QueryTemplate template = factory.createTemplate(queryString);
        addParameters(template);
        template.setResultsPerPage(parameters.getResultsPerPage());
        template.setPage(parameters.getPage());
        template.setDepth(parameters.getDepth());

        return template;
    }

    private String constructQueryString() {
        StringBuilder builder = new StringBuilder();
        builder.append("SELECT " + CI_SELECTOR_NAME + ".* FROM [" + CONFIGURATION_ITEM_NODETYPE_NAME + "] AS " + CI_SELECTOR_NAME);
        builder.append(joins.toString());

        if (!conditions.isEmpty()) {
            builder.append(" WHERE ");
            builder.append(Joiner.on(" AND ").join(Collections2.transform(conditions, new Function<Condition, String>() {
                @Override
                public String apply(final Condition input) {
                    return input.build();
                }
            })));
        }

        builder.append(" ORDER BY NAME(" + CI_SELECTOR_NAME + ")");

        logger.trace("Query string built: {}", builder.toString());

        return builder.toString();
    }

    private void addParameters(QueryTemplate query) {
        for (Condition condition : conditions) {
            if (condition.parameter == null)
                continue;

            for (int i = 0; i < condition.values.length; i++) {
                String key = condition.parameter + i;
                Object value = condition.values[i];

                query.setParameter(key, value);
                logger.trace("Bound {} to {}", key, value);
            }
        }
    }

    private void createJoinsAndConditions() {
        createConditionForParent();
        createConditionForAncestor();
        createConditionForName();
        createConditionForConfigurationItemTypeName();
        createConditionForDate();
        createConditionsForProperties();
        createConditionsForTextSearch();
    }

    private void createConditionForConfigurationItemTypeName() {
        if (parameters.getType() != null) {
            List<String> types = Lists.newArrayList();
            types.add(parameters.getType().toString());
            for (Type subtype : DescriptorRegistry.getSubtypes(parameters.getType())) {
                types.add(subtype.toString());
            }
            conditions.add(Condition.from(CI_SELECTOR_NAME, CONFIGURATION_ITEM_TYPE_PROPERTY_NAME, "_configurationItemTypeName", EQUALITY_OPERATOR, false,
                    (Object[]) types.toArray(new Object[types.size()])));
        }
    }

    private void createConditionForParent() {
        if (!Strings.nullToEmpty(parameters.getParent()).trim().isEmpty()) {
            String parent = Strings.nullToEmpty(parameters.getParent());
            if (!parent.startsWith("/")) {
                parent = "/" + parent;
            }
            conditions.add(Condition.isChildNode(CI_SELECTOR_NAME, parent));
        }
    }

    private void createConditionForAncestor() {
        if (!Strings.nullToEmpty(parameters.getAncestor()).trim().isEmpty()) {
            String ancestor = Strings.nullToEmpty(parameters.getAncestor());
            if (!ancestor.startsWith("/")) {
                ancestor = "/" + ancestor;
            }
            conditions.add(Condition.isDescendantNode(CI_SELECTOR_NAME, ancestor));
        }
    }

    private void createConditionForName() {
        if (!Strings.nullToEmpty(parameters.getName()).trim().isEmpty()) {
            if (parameters.getName().contains("%")) {
                conditions.add(Condition.nameLike(CI_SELECTOR_NAME, parameters.getName()));
            } else {
                conditions.add(Condition.nameIs(CI_SELECTOR_NAME, parameters.getName()));
            }
        }
    }

    private void createConditionForDate() {
        if (parameters.getBefore() != null) {
            conditions.add(Condition.from(CI_SELECTOR_NAME, LAST_MODIFIED_AT_PROPERTY_NAME, "_before", "<=", parameters.getBefore()));
        }
    }

    private void createConditionsForProperties() {
        for (Map.Entry<String, String> entry : parameters.getProperties().entrySet()) {
            createConditionForProperty(entry.getKey(), entry.getValue());
        }
    }

    private void createConditionForProperty(final String propertyName, final String propertyValue) {
        if (parameters.getType() != null) {
            Descriptor descriptor = DescriptorRegistry.getDescriptor(parameters.getType());
            if (descriptor != null) {
                PropertyDescriptor propertyDescriptor = descriptor.getPropertyDescriptor(propertyName);
                if (propertyDescriptor != null) {
                    if (propertyDescriptor.getKind() == CI) {
                        String selectorName = "referenced" + nextSelectorId;
                        joins.append(" INNER JOIN [" + CONFIGURATION_ITEM_NODETYPE_NAME + "] AS " + selectorName + " ON " + CI_SELECTOR_NAME + ".["
                                + propertyName + "] = " + selectorName + ".[jcr:uuid]");
                        int lastSlash = propertyValue.lastIndexOf('/');
                        checkArgument(lastSlash != -1, propertyValue + " is a ID but does not contain a slash (/)");
                        final String parent = getAbsolutePathFromId(propertyValue.substring(0, lastSlash));
                        final String name = propertyValue.substring(lastSlash + 1);
                        conditions.add(Condition.isChildNode(selectorName, parent));
                        conditions.add(Condition.nameIs(selectorName, name));
                        nextSelectorId++;
                        return;
                    } else if (propertyDescriptor.getKind() == SET_OF_CI || propertyDescriptor.getKind() == LIST_OF_CI) {
                        // FIXME: Find out how to make the JOIN works on this
                        // case. The INNER JOIN used for the single CI property
                        // causes unpredictable results.
                        // The results of the JOIN are different every time if
                        // the multi-value property has more than one value.
                        throw new IllegalArgumentException("Cannot query property " + propertyDescriptor + " because it is of type SET_OF_CIS");
                    }
                }
            }
        }
        conditions.add(Condition.match(CI_SELECTOR_NAME, propertyName, propertyName, propertyValue));
    }

    private void createConditionsForTextSearch() {
        int count = 0;
        for (Map.Entry<String, String> query : parameters.getTextSearchQueries().entrySet()) {
            conditions.add(Condition.textSearch(query.getKey(), query.getValue(), "textSearch" + count + "_"));
        }
    }

    private static class Condition {
        String selector;
        String field;
        String operator;
        String parameter;
        Object[] values;
        boolean caseInsensitive;

        private static final String NAME_IS_FORMAT = "NAME(%s) = \'%s\'";
        private static final String NAME_LIKE_FORMAT = "LOWER(NAME(%s)) LIKE \'%s\'";
        private static final String ISCHILDNODE_FORMAT = ISCHILDNODE_OPERATOR + "(%s, [\'%s\'])";
        private static final String ISDESCENDANTNODE_FORMAT = ISDESCENDANTNODE_OPERATOR + "(%s, [\'%s\'])";
        private static final String CONTAINS_FORMAT = "contains(%s.[%s], $%s)";
        private static final String CASE_SENSITIVE = "%s.[%s] %s $%s";
        private static final String CASE_INSENSITIVE = "LOWER(%s.[%s]) %s $%s";

        String build() {
            StringBuilder conditionString = new StringBuilder();
            conditionString.append("(");
            for (int i = 0; i < values.length; i++) {
                if (i > 0) {
                    conditionString.append(" OR ");
                }

                if (operator.equals(NAME_IS_OPERATOR)) {
                    conditionString.append(format(NAME_IS_FORMAT, selector, escapeSingleQuote(values[i])));
                } else if (operator.equals(NAME_LIKE_OPERATOR)) {
                    conditionString.append(format(NAME_LIKE_FORMAT, selector, escapeSingleQuote(values[i])));
                } else if (operator.equals(ISCHILDNODE_OPERATOR)) {
                    conditionString.append(format(ISCHILDNODE_FORMAT, selector, escapeSingleQuote(values[i])));
                } else if (operator.equals(ISDESCENDANTNODE_OPERATOR)) {
                    conditionString.append(format(ISDESCENDANTNODE_FORMAT, selector, escapeSingleQuote(values[i])));
                } else if (operator.equals(CONTAINS_OPERATOR)) {
                    conditionString.append(format(CONTAINS_FORMAT, selector, field, parameter + i));
                } else {
                    conditionString.append(format(caseInsensitive ? CASE_INSENSITIVE : CASE_SENSITIVE, selector, field, operator, parameter + i));
                }
            }
            conditionString.append(")");
            return conditionString.toString();
        }

        private String escapeSingleQuote(Object value) {
            return ((String) value).replaceAll("'", "''");
        }

        static Condition match(final String selector, final String field, final String parameter, final String value) {
            String operator;
            boolean caseInsensitive;
            if (value.contains("%")) {
                operator = LIKE_OPERATOR;
                caseInsensitive = true;
            } else {
                operator = EQUALITY_OPERATOR;
                caseInsensitive = false;
            }
            return from(selector, field, parameter, operator, caseInsensitive, value);
        }

        static Condition nameIs(final String selector, final String value) {
            return from(selector, null, null, NAME_IS_OPERATOR, false, value);
        }

        static Condition nameLike(final String selector, final String value) {
            return from(selector, null, null, NAME_LIKE_OPERATOR, false, value.toLowerCase());
        }

        static Condition isChildNode(final String selector, final String value) {
            return from(selector, null, null, ISCHILDNODE_OPERATOR, false, value);
        }

        static Condition isDescendantNode(final String selector, final String value) {
            return from(selector, null, null, ISDESCENDANTNODE_OPERATOR, false, value);
        }

        static Condition textSearch(String property, String textQuery, String parameterName) {
            return Condition.from(CI_SELECTOR_NAME, property, parameterName, CONTAINS_OPERATOR, false, textQuery);
        }

        static Condition from(final String selector, final String field, final String parameter, final String operator, final Object value) {
            return from(selector, field, parameter, operator, false, value);
        }

        static Condition from(final String selector, final String field, final String parameter, final String operator, final boolean caseInsensitive,
                              final Object... values) {
            Condition t = new Condition();
            t.selector = selector;
            t.field = field;
            t.parameter = parameter;
            t.operator = operator;
            t.caseInsensitive = caseInsensitive;
            t.values = values;
            return t;
        }
    }

    private static final Logger logger = LoggerFactory.getLogger(SearchQueryBuilder.class);
}
